%
% Comparacao entre metodos de discretizacao no dominio da frequencia
%
%
% Definicao do controlador PID H(s)
%
%                     1        Td*s
%   H(s) = Kp*( 1 + ----- + ------------ )    
%                   Ti*s     (Td/N)*s+1
% 
%   Utiliza-se o controlador PID definido em tempo discreto
%   onde a parte integral IP e a parte derivativa DP sao obtidas
%   atraves de algum metodo de discretizacao
%   A parte derivativa possui um filtro de 1a. ordem
%   com polo em s=(-N/Td)
%-------------------
% Constantes do controlador
% Kp = 0.5
% Ti = 20
% Td = 0.2 % 0.2
% N =  20

Kp = 1
Ti = 0.1
Td = 0.0 % 0.2
N =  20


% Kp = 3
% Ti = 0.1
% Td = 0.0% 0.2
% N =  20
% Frequencia de amostragem fa e intervalo de amostragem Ta
fa = 20
Ta = 1/fa
% Definicao de s e z como funcoes de transferencia
s = tf('s');
z = tf('z',Ta);
%
% Metodo FD - Forward-Difference s = (z-1)/Ta 
%
% Parte integral
IPFD = Ta/(Ti*(z-1));
% Parte derivativa
DPFD = (Td*(z-1)/Ta)*1/((Td*(z-1)/(N*Ta))+1);
Hfd = Kp*(1+IPFD+DPFD)
%
% Metodo BD - Backward-Difference s = (z-1)/(z*Ta)
%
% Parte integral
IPBD = (Ta*z)/(Ti*(z-1));
% Parte derivativa
DPBD = (Td*(z-1)/(z*Ta))*1/((Td*(z-1)/(z*N*Ta))+1)
Hbd = Kp*(1+IPBD+DPBD)
%
% Metodo Tustin, Transformacao Bilinear
%
% Dominio do tempo continuo
H = Kp*(1 +1/(Ti*s)+(Td*s)/((Td/N)*s+1))
% Dominio do tempo discreto, utilizacao da funcao c2d
Ht = c2d(H,Ta,'tustin')
%
% Metodo do casamento de polos e zeros - 'matched'
% Polos e zeros sao mapeados atraves da expressao z=exp(s*Ta)
% O ganho e' ajustado convenientemente
%
% Utilizacao da funcao c2d
Hm = c2d(H,Ta,'matched')
%
% logspace(fi,ff) gera um vetor que define um espaco logaritmico de 10^fi ate
% 10^ff
%
fi = -1;
ff = 2;
w = logspace(fi,ff);
% H(s) - dominio do tempo continuo
% Hfd(z) - forward-difference
% Hbd(z) - backward-difference
% Ht(z)  - Tustin, Transformacao bilinear
% Hm(z)  - Casamento de polos e zeros
bode(H,'k--',Hfd,'r-.',Hbd,'b-',Ht,'y--',Hm,'g-',w)
grid on
