%clear all
function [vetorrk,vetoruk,vetoryk] = sistemadecontrolePI
Fa = 20;         % Sampling frequency
Ta = 1/Fa;       % Sampling time
Duration = 50;   % Duracao em segundos
NumberOfTasksToExecute = round(Duration/Ta)

% Create and configure timer object
tm = timer('ExecutionMode','fixedRate', ...            % Run continuously
    'Period',Ta, ...                                   % Period = sampling time
    'TasksToExecute',NumberOfTasksToExecute, ...       % Runs NumberOfTasksToExecute times
    'TimerFcn',@MyTimerFcn, ...                        % Run MyTimerFcn at each timer event
    'StopFcn',@StopEverything);

% setup da placa
s = daq.createSession('ni');
addAnalogInputChannel(s,'Dev1',0:2,'Voltage')
addAnalogOutputChannel(s,'Dev1',0,'Voltage')
s.Rate = Fa;
%
% Como setar os parametros
%
% Parametros do controlador PID
%         b1*z^2 + b2*z + b3
% H(z) = --------------------
%         a1*z^2 + a2*z + a3
%
% Controlador PI
%         b1*z + b2
% H(z) = -----------
%         a1*z + a2
% fazer b3=0 a3=0
%
% Coeficientes correspondem a Kp=1.0 Ti=0.1 fa=20Hz
a1 =  1.0;
a2 = -1.0;
a3 = 0.0;
b1 =  1.25;
b2 = -0.75;
b3 =  0.0;
% Controlador P
%
% H(z) = b1 = Kp
% fazer a1=1, a2=0, a3=0, b2=0, b3=0
% 
% a1 =  1.0;
% a2 =  0.0;
% a3 =  0.0;
% b1 =  1.0; % Kp
% b2 =  0.0;
% b3 =  0.0;

% inicializacao de variaveis do sistema de controle
rk   = 0.0;
yk   = 0.0;
%
uk   = 0.0;
uk_1 = 0.0;
uk_2 = 0.0;
%
ek   = 0.0;
ek_1 = 0.0;
ek_2 = 0.0;

% Inicializacao do vetor que guarda o historico das variaveis
vetorrk = zeros(NumberOfTasksToExecute+1,1);
vetorrk(1) = 0; % instante 0 -> k=1
vetorrk(2) = 0;
vetoryk = zeros(NumberOfTasksToExecute+1,1);
vetoryk(1) = 0;
vetoryk(2) = 0;
vetoruk = zeros(NumberOfTasksToExecute+1,1);
vetoruk(1) = 0;
vetoruk(2) = 0;

% incializacao da variavel do tempo discreto
k=3;

% Start the timer
start(tm);

% Funcao que e' executada no timer tm periodicamente a cada Ta
function MyTimerFcn(~,~)    
% Leitura da Porta A/D
sample = inputSingleScan(s)     % le o dado do canal de entrada 0    
rk = sample(1);                 % Sinal de referencia gerador de funcoes
% Escolher aqui ou velocidade angular ou posicao angular
yk = sample(2);   % Velocidade angular
%yk = sample(3);   % Posicao angular - Potenciometro
%
% calculo do erro e(k)
%
ek = rk - yk;    
% Calculo do controle
uk = (-a2/a1)*uk_1+(-a3/a1)*uk_2+(b1/a1)*ek+(b2/a1)*ek_1+(b3/a1)*ek_2;   

% Controle de Saturacao
if uk >= 5.0 
   uk = 5.0;
else if uk <= -5.0
   uk = -5.0;
    end
end % fim da saturacao
    
% Escrita na Porta D/A - u(k)
outputSingleScan(s,uk);

% Salva os valores
vetorrk(k) = rk;
vetoryk(k) = yk;
vetoruk(k) = uk;
  
% Update de variaveis    
k = k+1;
uk_2 = uk_1; 
uk_1 = uk;
ek_2 = ek_1;   
ek_1 = ek;

end % Fim da funcao MyTymerFcn

% Funcao executada ao final da execucao do timer tm
function StopEverything(~,~)
    release(s);
    nsamples = length(vetorrk);
    t=0.0:Ta:(nsamples-1)*Ta;
    plot(t,vetorrk,'-',t,vetoruk,'-.',t,vetoryk,'--');
    grid on
    title('Referencia r(k) (-)    /    Esforco de controle u(k) (-.)    /    Saida da planta y(k) (--)')
    xlabel('tempo (s)');
    ylabel('tensao (Volts)');
    mensagem='acabou'
end % Fim de StopEverything

end     % function controlador

% Copyright 2005 The MathWorks, Inc
